@@ -19,6 +19,7 @@
 pub use crate::errors::{CapacityError, PushCapacityError};
 pub use crate::iterators::*;
 pub use crate::trait_impls::*;
+use crate::utils::reverse_copy;
 use core::cmp::{Ord, PartialEq};
 use core::intrinsics;
 use core::marker::PhantomData;
@@ -45,7 +46,8 @@ mod trait_impls;
 #[doc(hidden)]
 pub mod utils;
 
-/// A [`Vec`](alloc::vec::Vec)-like struct (mostly directly API-compatible where it can be)
+/// A [`Vec`](https://doc.rust-lang.org/nightly/alloc/vec/struct.Vec.html)-like struct
+/// (mostly directly API-compatible where it can be)
 /// implemented with const generics around an array of fixed `N` capacity.
 pub struct StaticVec<T, const N: usize> {
   data: MaybeUninit<[T; N]>,
@@ -103,9 +105,9 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// let v2 = StaticVec::<i32, 3>::new_from_array([1, 2, 3, 4, 5, 6]);
   /// assert_eq!(v2, [1, 2, 3]);
   /// ```
-  /// Note that StaticVec also implements [`From`](std::convert::From) for both slices and static
-  /// arrays, which may prove more ergonomic in some cases as it allows for a greater degree of
-  /// type inference:
+  /// Note that StaticVec also implements [`From`](https://doc.rust-lang.org/nightly/std/convert/trait.From.html)
+  /// for both slices and static arrays, which may prove more ergonomic in some cases as it allows
+  /// for a greater degree of type inference:
   /// ```
   /// // The StaticVec on the next line is inferred to be of type `StaticVec<&'static str, 4>`.
   /// let v = StaticVec::from(["A", "B", "C", "D"]);
@@ -138,10 +140,10 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// as well.)
   ///
   /// Being `const` necessitates that this function can only accept arrays with a length
-  /// exactly equivalent to the declared capacity of the resulting StaticVec, so if you do need
+  /// exactly equal to the declared capacity of the resulting StaticVec, so if you do need
   /// flexibility with regards to input lengths it's recommended that you use
-  /// [`new_from_array`](crate::StaticVec::new_from_array) or the [`From`](std::convert::From)
-  /// implementations instead.
+  /// [`new_from_array`](crate::StaticVec::new_from_array) or the
+  /// [`From`](https://doc.rust-lang.org/nightly/std/convert/trait.From.html) implementations instead.
   ///
   /// Note that both forms of the [`staticvec!`] macro are implemented using
   /// [`new_from_const_array`](crate::StaticVec::new_from_const_array), so you may also prefer
@@ -154,72 +156,12 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
   }
 
-  /// Returns a new StaticVec instance filled with the return value of an initializer function.
-  /// The length field of the newly created StaticVec will be equal to its capacity.
-  ///
-  /// Example usage:
-  /// ```
-  /// let mut i = 0;
-  /// let v = StaticVec::<i32, 64>::filled_with(|| { i += 1; i });
-  /// assert_eq!(v.len(), 64);
-  /// assert_eq!(v[0], 1);
-  /// assert_eq!(v[1], 2);
-  /// assert_eq!(v[2], 3);
-  /// assert_eq!(v[3], 4);
-  /// ```
-  #[inline]
-  pub fn filled_with<F>(mut initializer: F) -> Self
-  where F: FnMut() -> T {
-    let mut res = Self::new();
-    // You might think it would make more sense to use `push_unchecked` here.
-    // Originally, I did also! However, as of today (November 19, 2019), doing so
-    // both in this function and several others throughout the crate inhibits the ability
-    // of `rustc` to fully unroll and autovectorize various constant-bounds loops. If this changes
-    // in the future, feel free to open a PR switching out the manual code for `get_unchecked`, if
-    // you happen to notice it before I do.
-    for i in 0..N {
-      unsafe {
-        res.mut_ptr_at_unchecked(i).write(initializer());
-        res.length += 1;
-      }
-    }
-    res
-  }
-
-  /// Returns a new StaticVec instance filled with the return value of an initializer function.
-  /// Unlike for [`filled_with`](crate::StaticVec::filled_with), the initializer function in
-  /// this case must take a single usize variable as an input parameter, which will be called
-  /// with the current index of the `0..N` loop that
-  /// [`filled_with_by_index`](crate::StaticVec::filled_with_by_index) is implemented with
-  /// internally. The length field of the newly created StaticVec will be equal to its capacity.
-  ///
-  /// Example usage:
-  /// ```
-  /// let v = StaticVec::<usize, 64>::filled_with_by_index(|i| { i + 1 });
-  /// assert_eq!(v.len(), 64);
-  /// assert_eq!(v[0], 1);
-  /// assert_eq!(v[1], 2);
-  /// assert_eq!(v[2], 3);
-  /// assert_eq!(v[3], 4);
-  /// ```
-  #[inline]
-  pub fn filled_with_by_index<F>(mut initializer: F) -> Self
-  where F: FnMut(usize) -> T {
-    let mut res = Self::new();
-    for i in 0..N {
-      unsafe {
-        res.mut_ptr_at_unchecked(i).write(initializer(i));
-        res.length += 1;
-      }
-    }
-    res
-  }
-
   /// Returns the current length of the StaticVec.
-  /// Just as for a normal [`Vec`](alloc::vec::Vec), this means the number of elements that
-  /// have been added to it with [`push`](crate::StaticVec::push),
-  /// [`insert`](crate::StaticVec::insert), etc. except in the case that it has been set directly
-  /// with the unsafe [`set_len`](crate::StaticVec::set_len) function.
+  /// Just as for a normal [`Vec`](https://doc.rust-lang.org/nightly/alloc/vec/struct.Vec.html),
+  /// this means the number of elements that have been added to it with
+  /// [`push`](crate::StaticVec::push), [`insert`](crate::StaticVec::insert), etc. except in the
+  /// case that it has been set directly with the unsafe [`set_len`](crate::StaticVec::set_len)
+  /// function.
   #[inline(always)]
   pub const fn len(&self) -> usize {
     self.length
@@ -250,6 +192,14 @@ impl<T, const N: usize> StaticVec<T, N> {
     N - self.length
   }
 
+  /// Returns the total size of the inhabited part of StaticVec (which may be zero if it has a
+  /// length of zero or contains ZSTs) in bytes. Specifically, the return value of this function
+  /// amounts to a calculation of `size_of::<T>() * self.length`.
+  #[inline(always)]
+  pub const fn size_in_bytes(&self) -> usize {
+    intrinsics::size_of::<T>() * self.length
+  }
+
   /// Directly sets the length field of the StaticVec to `new_len`. Useful if you intend
   /// to write to it solely element-wise, but marked unsafe due to how it creates
   /// the potential for reading from uninitialized memory later on.
@@ -297,13 +247,14 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Returns a constant pointer to the first element of the StaticVec's internal array.
   #[inline(always)]
   pub const fn as_ptr(&self) -> *const T {
+    // Written like this so it can be `const fn`.
     &self.data as *const _ as *const T
   }
 
   /// Returns a mutable pointer to the first element of the StaticVec's internal array.
   #[inline(always)]
   pub fn as_mut_ptr(&mut self) -> *mut T {
-    &mut self.data as *mut _ as *mut T
+    self.data.as_mut_ptr() as *mut T
   }
 
   /// Returns a constant reference to a slice of the StaticVec's inhabited area.
@@ -687,22 +638,70 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Returns a separate, reversed StaticVec of the contents of the StaticVec's
   /// inhabited area without modifying the original data.
   /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to avoid soundness issues.
-  #[inline]
+  #[inline(always)]
   pub fn reversed(&self) -> Self
   where T: Copy {
     Self {
-      data: unsafe {
-        let mut res = Self::new_data_uninit();
-        self
-          .as_ptr()
-          .copy_to_nonoverlapping(res.as_mut_ptr() as *mut T, self.length);
-        res.get_mut().get_unchecked_mut(0..self.length).reverse();
-        res
-      },
+      data: reverse_copy(&self.data),
       length: self.length,
     }
   }
 
+  /// Returns a new StaticVec instance filled with the return value of an initializer function.
+  /// The length field of the newly created StaticVec will be equal to its capacity.
+  ///
+  /// Example usage:
+  /// ```
+  /// let mut i = 0;
+  /// let v = StaticVec::<i32, 64>::filled_with(|| { i += 1; i });
+  /// assert_eq!(v.len(), 64);
+  /// assert_eq!(v[0], 1);
+  /// assert_eq!(v[1], 2);
+  /// assert_eq!(v[2], 3);
+  /// assert_eq!(v[3], 4);
+  /// ```
+  #[inline]
+  pub fn filled_with<F>(mut initializer: F) -> Self
+  where F: FnMut() -> T {
+    let mut res = Self::new();
+    for i in 0..N {
+      unsafe {
+        res.mut_ptr_at_unchecked(i).write(initializer());
+        res.length += 1;
+      }
+    }
+    res
+  }
+
+  /// Returns a new StaticVec instance filled with the return value of an initializer function.
+  /// Unlike for [`filled_with`](crate::StaticVec::filled_with), the initializer function in
+  /// this case must take a single usize variable as an input parameter, which will be called
+  /// with the current index of the `0..N` loop that
+  /// [`filled_with_by_index`](crate::StaticVec::filled_with_by_index) is implemented with
+  /// internally. The length field of the newly created StaticVec will be equal to its capacity.
+  ///
+  /// Example usage:
+  /// ```
+  /// let v = StaticVec::<usize, 64>::filled_with_by_index(|i| { i + 1 });
+  /// assert_eq!(v.len(), 64);
+  /// assert_eq!(v[0], 1);
+  /// assert_eq!(v[1], 2);
+  /// assert_eq!(v[2], 3);
+  /// assert_eq!(v[3], 4);
+  /// ```
+  #[inline]
+  pub fn filled_with_by_index<F>(mut initializer: F) -> Self
+  where F: FnMut(usize) -> T {
+    let mut res = Self::new();
+    for i in 0..N {
+      unsafe {
+        res.mut_ptr_at_unchecked(i).write(initializer(i));
+        res.length += 1;
+      }
+    }
+    res
+  }
+
   /// Copies and appends all elements, if any, of a slice (which can also be `&mut` as it will
   /// coerce implicitly to `&`) to the StaticVec. If the slice has a length greater than the
   /// StaticVec's remaining capacity, any contents after that point are ignored.
@@ -1014,6 +1013,22 @@ impl<T, const N: usize> StaticVec<T, N> {
     res
   }
 
+  /// A concept borrowed from the widely-used `SmallVec` crate, this function
+  /// returns a tuple consisting of a constant pointer to the first element of the StaticVec,
+  /// the length of the StaticVec, and the capacity of the StaticVec.
+  #[inline(always)]
+  pub const fn triple(&self) -> (*const T, usize, usize) {
+    (self.as_ptr(), self.length, N)
+  }
+
+  /// A mutable version of [`triple`](crate::StaticVec::triple). This implementation differs from
+  /// the one found in `SmallVec` in that it only provides the first element of the StaticVec as
+  /// a mutable pointer, not also the length as a mutable reference.
+  #[inline(always)]
+  pub fn triple_mut(&mut self) -> (*mut T, usize, usize) {
+    (self.as_mut_ptr(), self.length, N)
+  }
+
   #[doc(hidden)]
   #[inline(always)]
   pub(crate) const fn new_data_uninit() -> MaybeUninit<[T; N]> {
