@@ -848,6 +848,18 @@ fn reversed() {
   assert!(v == [3, 2, 1]);
 }
 
+#[test]
+fn size_in_bytes() {
+  let x = StaticVec::<u8, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
+  assert_eq!(x.size_in_bytes(), 8);
+  let y = StaticVec::<u16, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
+  assert_eq!(y.size_in_bytes(), 16);
+  let z = StaticVec::<u32, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
+  assert_eq!(z.size_in_bytes(), 32);
+  let w = StaticVec::<u64, 8>::from([1, 2, 3, 4, 5, 6, 7, 8]);
+  assert_eq!(w.size_in_bytes(), 64);
+}
+
 #[test]
 fn set_len() {
   let mut v = staticvec![1, 2, 3];
@@ -908,6 +920,21 @@ fn swap_remove() {
   assert_eq!(v, ["baz", "qux"]);
 }
 
+#[test]
+fn triple() {
+  static V: StaticVec<usize, 4> = staticvec![4, 5, 6, 7];
+  assert_eq!(V.triple(), (V.as_ptr(), 4, 4));
+}
+
+#[test]
+fn triple_mut() {
+  let mut v = staticvec![4, 5, 6, 7];
+  let t = v.triple_mut();
+  assert_eq!(t, (v.as_mut_ptr(), 4, 4));
+  unsafe { *t.0 = 8 };
+  assert_eq!(v, [8, 5, 6, 7]);
+}
+
 #[test]
 fn truncate() {
   let mut vec = staticvec![1, 2, 3, 4, 5];
